@@ -5,7 +5,8 @@
 
 TODO! Sorry!
 
-                [@]SourceFileOrURL  Alg     Digest  Length   ModTime   TargetFileOrURL
+[@]SourceFileOrURL | Alg | Digest | Length | ModTime | TargetFileOrURL
+
 TOKEN NUMBER:    1                  2       3       4        5         6
 
 """
@@ -60,12 +61,35 @@ class NotFound(Exception):
         """
         return self.context.__str__()
 
-class CheckmReporter(object):
-    COLUMN_NAMES = [u'# [@]SourceFileOrURL',u'Alg',u'Digest',u'Length',u'ModTime']
-    def __init__(self):
+class CantOpen(Exception):
+    """The item or directory was either not found, or not accessible."""
+    def __init__(self, *arg, **kw):
+        """
+        FIXME
+        @param *arg:
+        @type *arg:
+        @param **kw:
+        @type **kw:
+        """
+        self.context = (arg, kw)
+    def __repr__(self):
         """
         FIXME
         """
+        return self.context.__str__()
+    def __str__(self):
+        """
+        FIXME
+        """
+        return self.context.__str__()
+
+class CheckmReporter(object):
+    COLUMN_NAMES = [u'# [@]SourceFileOrURL',u'Alg',u'Digest',u'Length',u'ModTime', u'TargetFileOrURL']
+    def __init__(self, flush_each_line = False):
+        """
+        FIXME
+        """
+        self.fl = flush_each_line
         self.scanner = CheckmScanner()
 
     def _get_max_len(self, report):
@@ -91,10 +115,9 @@ class CheckmReporter(object):
         """
         spaced_line = []
         for index in xrange(len(line)):
-            spaced_line.append(line[index])
             spaces = col_maxes[index]-len(line[index])+4
-            spaced_line.append(u" "*spaces)
-        return u"".join(spaced_line)
+            spaced_line.append(line[index] + u" "*spaces)
+        return u" | ".join(spaced_line)
 
     def create_bagit_manifest(self, scan_directory, algorithm, recursive=False, delimiter = "  ", filename=None):
         """
@@ -122,14 +145,21 @@ class CheckmReporter(object):
                 if line[2] != "d":
                     if os.path.abspath(line[0]) != os.path.abspath(faked_filename):
                         filename.write("%s%s%s\n" % (line[2], delimiter, line[0]))
+                        logger.debug("Scanned file %s with %s -- %s" % (line[0], line[1], line[2]))
+                        if self.fl:
+                            filename.flush()
                     else:
                         logger.info("Manifest file match - scan line ignored")
         else:
+            # TODO Check to see if file exists
             with codecs.open(filename, encoding='utf-8', mode="w") as output:
                 for line in report:
                     if line[2] != "d":
                         if os.path.abspath(line[0]) != os.path.abspath(filename):
                             output.write("%s%s%s\n" % (line[2], delimiter, line[0]))
+                            logger.debug("Scanned file %s with %s -- %s" % (line[0], line[1], line[2]))
+                            if self.fl:
+                                output.flush()
                         else:
                             logger.info("Manifest file match - scan line ignored")
                 output.write("\n")
@@ -161,6 +191,7 @@ class CheckmReporter(object):
                         line = self.scanner.scan_path(os.path.join(dirname, subdir, checkm_filename), algorithm, columns)
                         logger.info("Line - %s" % line)
                         line[0] = '@%s' % (line[0])
+                        logger.debug("Adding a reference to a checkm manifest in directory:%s" % (line[0]))
                         subdir_report.append(line)
                     except Exception, e:
                         print dirname, subdir, checkm_filename
@@ -168,6 +199,8 @@ class CheckmReporter(object):
                 col_maxes = self._get_max_len(subdir_report)
                 for line in subdir_report:
                     output.write('%s\n' % (self._space_line(line, col_maxes)))
+                    if self.fl:
+                        output.flush()
                 output.write('\n')
 
     def create_checkm_file(self, scan_directory, algorithm, checkm_filename, recursive=False, columns=3, checkm_file=None):
@@ -176,20 +209,23 @@ class CheckmReporter(object):
                                                                                           algorithm, columns))
         report = self.scanner.scan_directory(scan_directory, algorithm, recursive=recursive, columns=columns)
         col_maxes = self._get_max_len(report)
-        if checkm_file != None and hasattr(checkm_file, 'write'):
-            checkm_file.write("%s \n" % (self._space_line(CheckmReporter.COLUMN_NAMES[:columns], col_maxes)))
+        if checkm_filename != None and hasattr(checkm_filename, 'write'):
+            checkm_filename.write("%s \n" % (self._space_line(CheckmReporter.COLUMN_NAMES[:columns], col_maxes)))
             for line in report:
-                if os.path.abspath(line[0]) != os.path.abspath(checkm_filename):
-                    checkm_file.write("%s\n" % (self._space_line(line, col_maxes)))
-                else:
-                    logger.info("Manifest file match - scan line ignored")
-            return checkm_file
+                checkm_filename.write("%s\n" % (self._space_line(line, col_maxes)))
+                logger.debug("Scanned file %s with %s -- %s" % (line[0], line[1], line[2]))
+                if self.fl:
+                    checkm_filename.flush()
+            return checkm_filename
         else:
             with codecs.open(checkm_filename, encoding='utf-8', mode="w") as output:
                 output.write("%s \n" % (self._space_line(CheckmReporter.COLUMN_NAMES[:columns], col_maxes)))
                 for line in report:
                     if os.path.abspath(line[0]) != os.path.abspath(checkm_filename):
                         output.write("%s\n" % (self._space_line(line, col_maxes)))
+                        logger.debug("Scanned file %s with %s -- %s" % (line[0], line[1], line[2]))
+                        if self.fl:
+                            output.flush()
                     else:
                         logger.info("Manifest file match - scan line ignored")
                 output.write("\n")
@@ -225,7 +261,12 @@ class CheckmReporter(object):
                 except NotFound:
                     scan_row = "File not found"
                     logger.info("Failed original: %s" % row)
-                    logger.info("But file not found at this path.")
+                    logger.error("File not found at this path - '%s'" % row[1])
+                    results['fail'][row[1]] = (row, scan_row)
+                except CantOpen:
+                    scan_row = "File could not be opened to read. Check file permissions."
+                    logger.info("Failed original: %s" % row)
+                    logger.error("File could not be opened to read. Check file permissions - '%s'" % row[1])
                     results['fail'][row[1]] = (row, scan_row)
         return results
 
@@ -249,6 +290,7 @@ class CheckmReporter(object):
                         if not columns:
                             columns = len(row)
                         scan_row = scanner.scan_path(row[0], row[1], columns)
+                            
                         nomatch = False
                         for expected, scanned in zip(row, scan_row):
                             if expected != "-" and expected != scanned:
@@ -262,11 +304,16 @@ class CheckmReporter(object):
                     except NotFound:
                         scan_row = "File not found"
                         logger.info("Failed original: %s" % row)
-                        logger.info("But file not found at this path.")
+                        logger.error("File not found at this path - '%s'" % row[0])
                         results['fail'][row[0]] = (row, scan_row)
+                    except CantOpen:
+                        scan_row = "File could not be opened to read. Check file permissions."
+                        logger.info("Failed original: %s" % row)
+                        logger.error("File could not be opened to read. Check file permissions - '%s'" % row[0])
+                        results['fail'][row[1]] = (row, scan_row)
             return results
         
-        logger.info("Checking files against %s checkm manifest" % checkm_filename)
+        logger.info("Checking files against '%s' checkm manifest" % checkm_filename)
         parser = CheckmParser(checkm_filename)
         results = _check_files_against_parser(parser, columns)
         if ignore_multilevel:
@@ -280,11 +327,14 @@ class CheckmReporter(object):
                 additional_results = _check_files_against_parser(parser, columns)
                 # Add to the passes
                 results['pass'].extend(additional_results['pass'])
+                logger.debug("%s files from within %s passed the check" % (len(additional_results['pass']), checkm_file))
                 # add to the overall list of 
                 results['include'].extend(additional_results['include'])
+                logger.debug("Found %s additional checkm manifests within %s" % (len(additional_results['include']), checkm_file))
                 checkm_list.extend(additional_results['include'])
                 # add to the fail dict
                 results['fail'].update(additional_results['fail'])
+                logger.debug("Found %s failed their check within %s" % (len(additional_results['fail']), checkm_file))
             return results
 
 class BagitParser(object):
@@ -359,7 +409,7 @@ class BagitParser(object):
             """
             if not line.startswith('#'):
                 tokens = filter(lambda x: x, re.split("\s+", line, 1)) # 2 columns
-                logger.info(tokens)
+                logger.debug("Parsed line to: '%s'" % ("', '".join(tokens)))
                 if tokens:
                     # handle "\s*\*" situation
                     if tokens[1].startswith("*"):
@@ -426,11 +476,16 @@ class CheckmParser(object):
         @type checkm_file:
         """
         if not hasattr(checkm_file, "read"):
-            if os.path.isfile(checkm_file):
-                with codecs.open(checkm_file, encoding='utf-8', mode="r") as check_fh:
-                    self._parse_lines(check_fh)
-            else:
-                raise NotFound(checkm_file=checkm_file)
+            try:
+                if os.path.isfile(checkm_file):
+                    with codecs.open(checkm_file, encoding='utf-8', mode="r") as check_fh:
+                        self._parse_lines(check_fh)
+                else:
+                    raise NotFound(checkm_file=checkm_file)
+            except IOError, e:
+                raise CantOpen(checkm_file)
+            except OSError, e:
+                raise NotFound(checkm_file)
         else:
             self._parse_lines(checkm_file)
         return self.lines
@@ -450,8 +505,9 @@ class CheckmParser(object):
             @type line:
             """
             if not line.startswith('#'):
-                tokens = filter(lambda x: x, re.split("\s+", line, 5)) # 6 column max defn == 5 splits
-                logger.info(tokens)
+                tokens = filter(lambda x: x, re.split("\|", line, 5)) # 6 column max defn == 5 splits
+                tokens = [token.strip() for token in tokens]          # remove trailing and leading whitespace
+                logger.debug("Parsed line to: '%s'" % ("', '".join(tokens)))
                 if tokens:
                     self.lines.append(tokens)
 
@@ -476,7 +532,12 @@ class CheckmScanner(object):
         report = []
         for item in os.listdir(directory_path):
             item_path = os.path.join(directory_path, item)
-            report.append(self.scan_path(item_path, algorithm, columns))
+            try:
+                report.append(self.scan_path(item_path, algorithm, columns))
+            except CantOpen, e:
+                logger.error("Cannot open %s to check fixity" % item_path)
+            except NotFound, e:
+                logger.error("Cannot find %s to check fixity" % item_path)
         return report
 
     def scan_tree(self, directory_path, algorithm, columns):
@@ -492,8 +553,13 @@ class CheckmScanner(object):
         report = []
         if os.path.exists(directory_path):
             for (dirpath, dirnames, filenames) in os.walk(directory_path):
-                for item_path in [os.path.join(dirpath, x) for x in dirnames+filenames]:
-                    report.append(self.scan_path(item_path, algorithm, columns))
+                for item_path in [os.path.join(dirpath, x) for x in dirnames+filenames]:        
+                    try:
+                        report.append(self.scan_path(item_path, algorithm, columns))
+                    except CantOpen, e:
+                        logger.error("Cannot open %s to check fixity" % item_path)
+                    except NotFound, e:
+                        logger.error("Cannot find %s to check fixity" % item_path)
             return report
         else:
             raise NotFound(directory_path=directory_path, recursive=True)
@@ -537,11 +603,11 @@ class CheckmScanner(object):
                     line.append(unicode(os.stat(item_path)[ST_MTIME]))
             return line
         except OSError:
-            logger.info("item exists? %s" % os.path.exists(item_path))
+            logger.info("Had an OSError trying to open %s" % (item_path))
             raise NotFound(item_path=item_path)
         except IOError:
-            logger.info("item exists? %s" % os.path.exists(item_path))
-            raise NotFound(item_path=item_path)
+            logger.info("Had an IOError trying to open %s" % (item_path))
+            raise CantOpen(item_path=item_path)
         except AttributeError:
             raise ValueError("This tool cannot perform hashtype %s" % algorithm)
         
